/*******************************************************************************
* File Name: j1FreeRTOS_StatsCounter_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "j1FreeRTOS_StatsCounter.h"

static j1FreeRTOS_StatsCounter_backupStruct j1FreeRTOS_StatsCounter_backup;


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  j1FreeRTOS_StatsCounter_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_SaveConfig(void) 
{
    #if (!j1FreeRTOS_StatsCounter_UsingFixedFunction)

        j1FreeRTOS_StatsCounter_backup.CounterUdb = j1FreeRTOS_StatsCounter_ReadCounter();

        #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved)
            j1FreeRTOS_StatsCounter_backup.CounterControlRegister = j1FreeRTOS_StatsCounter_ReadControlRegister();
        #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved) */

    #endif /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  j1FreeRTOS_StatsCounter_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_RestoreConfig(void) 
{      
    #if (!j1FreeRTOS_StatsCounter_UsingFixedFunction)

       j1FreeRTOS_StatsCounter_WriteCounter(j1FreeRTOS_StatsCounter_backup.CounterUdb);

        #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved)
            j1FreeRTOS_StatsCounter_WriteControlRegister(j1FreeRTOS_StatsCounter_backup.CounterControlRegister);
        #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved) */

    #endif /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  j1FreeRTOS_StatsCounter_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_Sleep(void) 
{
    #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved)
        /* Save Counter's enable state */
        if(j1FreeRTOS_StatsCounter_CTRL_ENABLE == (j1FreeRTOS_StatsCounter_CONTROL & j1FreeRTOS_StatsCounter_CTRL_ENABLE))
        {
            /* Counter is enabled */
            j1FreeRTOS_StatsCounter_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            j1FreeRTOS_StatsCounter_backup.CounterEnableState = 0u;
        }
    #else
        j1FreeRTOS_StatsCounter_backup.CounterEnableState = 1u;
        if(j1FreeRTOS_StatsCounter_backup.CounterEnableState != 0u)
        {
            j1FreeRTOS_StatsCounter_backup.CounterEnableState = 0u;
        }
    #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved) */
    
    j1FreeRTOS_StatsCounter_Stop();
    j1FreeRTOS_StatsCounter_SaveConfig();
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  j1FreeRTOS_StatsCounter_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_Wakeup(void) 
{
    j1FreeRTOS_StatsCounter_RestoreConfig();
    #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved)
        if(j1FreeRTOS_StatsCounter_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            j1FreeRTOS_StatsCounter_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved) */
    
}


/* [] END OF FILE */
