//=====================================================================
//
//	j1Put.c - j1sys device independent put lib
//
//	version 1.2 - 15 jan 2015
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JOSHUA 1 SYSTEMS INC. OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include "j1Put.h"

//---------------------------------------------------------------------
//	j1Put_Start() - init the control
//---------------------------------------------------------------------

void j1Put_Start(j1Put_CTRL *pC, void *Put_Char, void *deviceCtrl)
{
	pC->Put_Char	= Put_Char;
	pC->deviceCtrl	= deviceCtrl;
}

//---------------------------------------------------------------------
//	j1Put_Char() - put a char using control
//---------------------------------------------------------------------

void j1Put_Char(j1Put_CTRL *pC, char c)
{
	if (pC->Put_Char) pC->Put_Char(pC->deviceCtrl, c);
}

//---------------------------------------------------------------------
//	j1Put_String() - simple iterative string via _Char
//---------------------------------------------------------------------

void j1Put_String(j1Put_CTRL *pC, char *string)
{
	char	c;
	
	if (pC->Put_Char)
	{
		while ((c = *string++)) pC->Put_Char(pC->deviceCtrl, c);
	}
}

//---------------------------------------------------------------------
//	j1Put_StringOpt() - simple iterative string via _Char w/options
//---------------------------------------------------------------------

void j1Put_StringOpt(j1Put_CTRL *pC, char *string, uint16 options)
{
	char	c;
	
	if (pC->Put_Char)
	{
		if (options & j1Put_PRESPC) pC->Put_Char(pC->deviceCtrl, ' ');
		while ((c = *string++)) pC->Put_Char(pC->deviceCtrl, c);
		if (options & j1Put_PSTSPC) pC->Put_Char(pC->deviceCtrl, ' ');
	}
}

//---------------------------------------------------------------------
//	j1Put_StringField() - string output limited/padded w/options
//---------------------------------------------------------------------

void j1Put_StringField(j1Put_CTRL *pC, char *string, uint16 length, uint16 options)
{
	char	c;
	
	if (pC->Put_Char)
	{
		if (options & j1Put_PRESPC) pC->Put_Char(pC->deviceCtrl, ' ');

		while ((c = *string++))
		{
			if (length)
			{
				pC->Put_Char(pC->deviceCtrl, c);
				length--;
			}
		}
		
		while (length--) pC->Put_Char(pC->deviceCtrl, ' ');

		if (options & j1Put_PSTSPC) pC->Put_Char(pC->deviceCtrl, ' ');
	}
}

//---------------------------------------------------------------------
//	j1Put_Field() - simple iterative field output w/options
//---------------------------------------------------------------------

void j1Put_Field(j1Put_CTRL *pC, char *field, uint16 length, uint16 options)
{
	if (pC->Put_Char)
	{
		if (options & j1Put_PRESPC) pC->Put_Char(pC->deviceCtrl, ' ');
		while (length--) pC->Put_Char(pC->deviceCtrl, *field++);
		if (options & j1Put_PSTSPC) pC->Put_Char(pC->deviceCtrl, ' ');
	}
}

//---------------------------------------------------------------------
//	j1Put_Hex8() - output 8 bit hex w/options
//---------------------------------------------------------------------

void j1Put_Hex8(j1Put_CTRL *pC, uint8 hex, uint16 options)
{
	char	nib;
	
	if (options & j1Put_PRESPC)
	{
		j1Put_Char(pC, ' ');
		options &= ~j1Put_PRESPC;
	}
	
	if (options & j1Put_HEX_PREFIX)
	{
		j1Put_String(pC, "0x");
		options &= ~j1Put_HEX_PREFIX;
	}
	
	nib = (hex >> 4) + '0';
	if (nib > '9')
	{
		if (options & j1Put_HEX_LOWER) nib += 'a' - '9' - 1;
								  else nib += 'A' - '9' - 1;
	}
	j1Put_Char(pC, nib);

	nib = (hex & 0x0f) + '0';
	if (nib > '9')
	{
		if (options & j1Put_HEX_LOWER) nib += 'a' - '9' - 1;
								  else nib += 'A' - '9' - 1;
	}
	j1Put_Char(pC, nib);
	
	if (options & j1Put_PSTSPC)
	{
		j1Put_Char(pC, ' ');
		options &= ~j1Put_PSTSPC;
	}
}

//---------------------------------------------------------------------
//	j1Put_Hex16() - output 16 bit hex w/options
//---------------------------------------------------------------------

void j1Put_Hex16(j1Put_CTRL *pC, uint16 hex, uint16 options)
{
	if (options & j1Put_PRESPC)
	{
		j1Put_Char(pC, ' ');
		options &= ~j1Put_PRESPC;
	}

	if (options & j1Put_HEX_PREFIX)
	{
		j1Put_String(pC, "0x");
		options &= ~j1Put_HEX_PREFIX;
	}
	
	j1Put_Hex8(pC, hex >> 8  , options & ~j1Put_PSTSPC);
	j1Put_Hex8(pC, hex & 0xff, options & ~j1Put_PSTSPC);
	
	if (options & j1Put_PSTSPC)
	{
		j1Put_Char(pC, ' ');
		options &= ~j1Put_PSTSPC;
	}
}

//---------------------------------------------------------------------
//	j1Put_Hex32() - output 32 bit hex w/options
//---------------------------------------------------------------------

void j1Put_Hex32(j1Put_CTRL *pC, uint32 hex, uint16 options)
{
	if (options & j1Put_PRESPC)
	{
		j1Put_Char(pC, ' ');
		options &= ~j1Put_PRESPC;
	}

	if (options & j1Put_HEX_PREFIX)
	{
		j1Put_String(pC, "0x");
		options &= ~j1Put_HEX_PREFIX;
	}

	j1Put_Hex16(pC, hex >> 16   , options & ~j1Put_PSTSPC);
	j1Put_Hex16(pC, hex & 0xffff, options & ~j1Put_PSTSPC);
	
	if (options & j1Put_PSTSPC)
	{
		j1Put_Char(pC, ' ');
		options &= ~j1Put_PSTSPC;
	}
}

//---------------------------------------------------------------------
//	j1Put_HexDump() - output hex dump line w/options
//---------------------------------------------------------------------

void j1Put_HexDump(j1Put_CTRL *pC, uint32 addrHex, uint8 *bfr, uint8 lo, uint8 hi, uint16 options)
{
	uint8	i, len, *bfrPtr, c;
	
	if (options & j1Put_HEXDUMPA32) j1Put_Hex32(pC, addrHex, options);
	else j1Put_Hex16(pC, (uint16) addrHex, options);
	
	if (options & j1Put_HEXDUMP16) len = 16;
	else len = 8;
	
	bfrPtr = bfr;
	for (i = 0; i < len; i++)
	{
		if (i < lo || hi < i) j1Put_StringOpt(pC, "..", j1Put_PRESPC);
		else j1Put_Hex8(pC, *bfrPtr++, options | j1Put_PRESPC);
	}
	
	if (options & j1Put_HEXDUMPASC)
	{
		j1Put_String(pC, " |");
		
		bfrPtr = bfr;
		for (i = 0; i < len; i++)
		{
			if (i < lo || hi < i) j1Put_Char(pC, ' ');
			else
			{
				c = *bfrPtr++;
				if (c < 0x20 || 0x7f < c) c = 0x20;
				j1Put_Char(pC, c);
			}
		}
		
		j1Put_Char(pC, '|');
	}
}

//---------------------------------------------------------------------
//	j1Put_FillRegion() - special shortcut to GFX
//---------------------------------------------------------------------

#if 0
void j1Put_FillRegion(j1Put_CTRL *pC, uint32 rgb)
{
	j1GFX_CTRL	*gC;
	
	if (pC)
	{
		gC = (j1GFX_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			j1GFX_FillRegion(gC, rgb);
		}
	}
}
#endif

//---------------------------------------------------------------------
//	j1Put_SetFontFgBg() - special shortcut to GFX
//---------------------------------------------------------------------

#if 0
void j1Put_SetFontFgBg(j1Put_CTRL *pC, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg)
{
	j1GFX_CTRL	*gC;
	
	if (pC)
	{
		gC = (j1GFX_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			j1GFX_SetFontFgBg(gC, font, scale, fg, bg);
		}
	}
}
#endif

//---------------------------------------------------------------------
//	j1Put_SetTextRowCol() - special shortcut to GFX
//---------------------------------------------------------------------

#if 0
void j1Put_SetTextRowCol(j1Put_CTRL *pC, uint8 row, uint8 col)
{
	j1GFX_CTRL	*gC;
	
	if (pC)
	{
		gC = (j1GFX_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			j1GFX_SetTextRowCol(gC, row, col);
		}
	}
}
#endif

//---------------------------------------------------------------------
//	j1Put_SetTextXY() - special shortcut to GFX
//---------------------------------------------------------------------

#if 0
void j1Put_SetTextXY(j1Put_CTRL *pC, uint16 x, uint16 y)
{
	j1GFX_CTRL	*gC;
	
	if (pC)
	{
		gC = (j1GFX_CTRL *) pC->deviceCtrl;
		if (gC)
		{
			j1GFX_SetTextXY(gC, x, y);
		}
	}
}
#endif
